@@ -643,11 +643,6 @@ bool PowerMac7_2_CPUFanCtrlLoop::updateMetaState( void )
 	}
 }
 
-void PowerMac7_2_CPUFanCtrlLoop::adjustControls( void )
-{
-	 // [4001211] Override to prevent calculateNewTarget() from being called from when the environment changes.
-}
-
 bool PowerMac7_2_CPUFanCtrlLoop::acquireSample( void )
 {
 	samplePoint * latest;
@@ -1217,14 +1212,11 @@ void PowerMac7_2_CPUFanCtrlLoop::deadlinePassed( void )
 
 	} // endif 9s
 
-	 // [4001211] Always calculate the PID function when the interval deadline expires.
-	if (ctrlloopState != kIOPCtrlLoopNotReady)
+	// If we changed the environment, the platform plugin will invoke updateMetaState()
+	// and adjustControls().  If not, then we just need to call adjustControls()
+	if (!didSetEnv)
 	{
-		// Apply the PID algorithm.
-		ControlValue newTarget = calculateNewTarget();
-
-		// set the target
-		sendNewTarget( newTarget );
+		adjustControls();
 	}
 
 	// set the deadline
