@@ -22,32 +22,23 @@
 /*
  * Copyright (c) 2004 Apple Computer, Inc.  All rights reserved.
  *
- *  File: $Id: PowerMac9_1_ThermalProfile.h,v 1.2 2004/04/23 01:29:36 murph Exp $
  */
 
 
-#ifndef _POWERMAC9_1_THERMALPROFILE_H
-#define _POWERMAC9_1_THERMALPROFILE_H
+#ifndef _POWERMAC11_2_THERMALPROFILE_H
+#define _POWERMAC11_2_THERMALPROFILE_H
 
 #include "IOPlatformPlugin.h"
 
 
-class PowerMac9_1_ThermalProfile : public IOPlatformPluginThermalProfile
+class PowerMac11_2_ThermalProfile : public IOPlatformPluginThermalProfile
 	{
-	OSDeclareDefaultStructors( PowerMac9_1_ThermalProfile )
-
-protected:
-
-	enum
-		{
-		kCPUPowerSensorID			= 0x05
-		};
+	OSDeclareDefaultStructors( PowerMac11_2_ThermalProfile )
 
 public:
-
-	virtual			UInt8			getThermalConfig( void );
-	virtual			void			adjustThermalProfile( void );
-	};
+	virtual	bool start( IOService * provider );
+	virtual	void adjustThermalProfile( void );
+};
 
 
-#endif // _POWERMAC9_1_THERMALPROFILE_H
+#endif // _POWERMAC11_2_THERMALPROFILE_H
