@@ -20,25 +20,39 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
- * Copyright (c) 2004 Apple Computer, Inc.  All rights reserved.
+ * Copyright (c) 2004-2005 Apple Computer, Inc.  All rights reserved.
  *
- *  File: $Id: PowerMac9_1_ThermalProfile.cpp,v 1.2 2004/04/23 01:29:36 murph Exp $
+ *  File: $Id: PowerMac8_1_ThermalProfile.cpp,v 1.7 2005/09/11 22:40:11 raddog Exp $
  */
 
 
 #include "IOPlatformPluginSymbols.h"
-#include "PowerMac9_1_ThermalProfile.h"
+#include "SMU_Neo2_PlatformPlugin.h"
 
-OSDefineMetaClassAndStructors( PowerMac9_1_ThermalProfile, IOPlatformPluginThermalProfile )
+#include "PowerMac8_1_ThermalProfile.h"
+
+OSDefineMetaClassAndStructors( PowerMac8_1_ThermalProfile, IOPlatformPluginThermalProfile )
 
 
-UInt8 PowerMac9_1_ThermalProfile::getThermalConfig( void )
+UInt8 PowerMac8_1_ThermalProfile::getThermalConfig( void )
 	{
-	return( 0 );
+	SMU_Neo2_PlatformPlugin*			smuPlatformPlugin;
+	UInt8								platformThermalModelID = 0;
+
+	smuPlatformPlugin = OSDynamicCast( SMU_Neo2_PlatformPlugin, platformPlugin );
+
+	if ( ( smuPlatformPlugin == NULL ) || !smuPlatformPlugin->getSDBPartitionData( kSensorTreePartID, 4, 1, &platformThermalModelID ) )
+		{
+		// If platformThermalModelID cannot be found, then default to an EVT Q45A.
+
+		platformThermalModelID = 0;
+		}
+
+	return( platformThermalModelID );
 	}
 
 
-void PowerMac9_1_ThermalProfile::adjustThermalProfile( void )
+void PowerMac8_1_ThermalProfile::adjustThermalProfile( void )
 	{
 	const OSNumber*						sensorID;
 	OSArray*							sensorInfoDicts;
@@ -48,16 +62,18 @@ void PowerMac9_1_ThermalProfile::adjustThermalProfile( void )
 	// they correspond to.  Nothing will ever register with their sensor ID, so we have to
 	// manually add them to the registry so that they show up in PlatformConsole.
 
-	sensorInfoDicts = OSDynamicCast( OSArray, platformPlugin->getProperty( gIOPPluginSensorDataKey ) );
+	sensorInfoDicts = platformPlugin->getSensorInfoDicts();
 
 	sensorID = OSNumber::withNumber( kCPUPowerSensorID, 32 );
 
 	if ( ( powerSensor = platformPlugin->lookupSensorByID( sensorID ) ) != NULL )
-	{
-		sensorInfoDicts->setObject( powerSensor->getInfoDict());
-	}
+		{
+		sensorInfoDicts->setObject( powerSensor->getInfoDict() );
+
+		// Publish a copy of sensorInfoDicts in the registry
+		platformPlugin->setSensorInfoDicts (sensorInfoDicts);
+		}
 
 	sensorID->release();
 
-	sensorInfoDicts->release();
 	}
